local *

{
  :AcceptResurrect
  :C_DeathInfo
  :C_PetJournal
  :C_SummonInfo
  :CanMerchantRepair
  :ConfirmSummon
  :Dismount
  :ERR_ATTACK_MOUNTED
  :ERR_MOUNT_ALREADYMOUNTED
  :ERR_NOT_WHILE_MOUNTED
  :ERR_TAXIPLAYERALREADYMOUNTED
  :GetCoinTextureString
  :GetContainerItemLink
  :GetContainerNumSlots
  :GetCorpseRecoveryDelay
  :GetItemInfo
  :GetMoney
  :GetRepairAllCost
  :IsInInstance
  :IsMounted
  :RepairAllItems
  :RepopMe
  :SPELL_FAILED_NOT_MOUNTED
  :StaticPopup1
  :StaticPopup1Button1
  :UnitAffectingCombat
  :UseContainerItem
  :idTweaks
  :idTweaksModule
  :idTweaksOptions
} = _G
{
  :Option
  :ToggleOption
} = idTweaksOptions

class Automation extends idTweaksModule
  mount_error_messages:
    [ERR_ATTACK_MOUNTED]: true
    [ERR_MOUNT_ALREADYMOUNTED]: true
    [ERR_NOT_WHILE_MOUNTED]: true
    [ERR_TAXIPLAYERALREADYMOUNTED]: true
    [SPELL_FAILED_NOT_MOUNTED]: true
  
  new: (addon) =>
    -- print "Automation#new"

    super addon, "Automation", "AceConsole-3.0", "AceEvent-3.0"

  set_options: =>
    -- print "Automation#set_options"

    super!

    @options\add_option Option @, "accepting", "Accepting", "Automatically accept certain offers.", {},
      accept_hearthstone_bind: ToggleOption @, "accept_hearthstone_bind", "Accept Hearthstone Bind", "Automatically accepts HearthStone binds"
      accept_pvp_release: ToggleOption @, "accept_pvp_release", "Accept PVP Release", "Automatically accepts PVP Release"
      accept_resurrection: ToggleOption @, "accept_resurrection", "Accept Resurrection", "Automatically accepts Resurrections"
      accept_summon: ToggleOption @, "accept_summon", "Accept Summon", "Automatically accepts Summons (disabled by default for raiders)"

    @options\add_option Option @, "vendors", "Vendors", "Automate Vendor Interactions.", {},
      repair_gear: ToggleOption @, "repair_gear", "Repair Gear", "Automatically repairs your gear"
      sell_grey_items: ToggleOption @, "sell_grey_items", "Sell Grey Items", "Automatically sells grey items"

    if idTweaks\is_retail!
      @options\add_option Option @, "minipet", "Minipet", "Minipet", {},
        summon: ToggleOption @, "summon", "Summon", "Summon Random Favorite Battlepet"

    if idTweaks\is_bc!
      @options\add_option Option @, "movement", "Movement", "Movement", {},
        dismount: ToggleOption @, "dismount", "Dismount", "Dismount when needed"

  enable: =>
    -- print "Automation#enable"

    @ace_module\RegisterEvent "CONFIRM_BINDER", (...) ->
      return unless @db.accepting.accept_hearthstone_bind
      @accept_hearthstone_bind!
      
    @ace_module\RegisterEvent "CONFIRM_SUMMON", (...) ->
      return unless @db.accepting.accept_summon
      return if UnitAffectingCombat "player"
      @accept_summon!

    @ace_module\RegisterEvent "MERCHANT_SHOW", (...) ->
      @sell_grey_items! if @db.vendors.sell_grey_items == true
      @repair_gear! if CanMerchantRepair! and @db.vendors.repair_gear == true

    @ace_module\RegisterEvent "PLAYER_DEAD", (...) ->
      return unless @db.accepting.accept_pvp_release
      return if UnitAffectingCombat "player"
      return if C_DeathInfo.GetSelfResurrectOptions! and #C_DeathInfo.GetSelfResurrectOptions! > 0
      @accept_pvp_release!

    @ace_module\RegisterEvent "RESURRECT_REQUEST", (...) ->
      return unless @db.accepting.accept_resurrection
      return if GetCorpseRecoveryDelay! ~= 0
      @accept_resurrection!
    
    if idTweaks\is_retail!
      @ace_module\RegisterEvent "PLAYER_ALIVE", (...) ->
        return unless idTweaks\is_retail!
        return unless @db.minipet.summon
        return if UnitAffectingCombat "player"
        @summon_battlepet!

      @ace_module\RegisterEvent "PLAYER_CONTROL_GAINED", (...) ->
        return unless idTweaks\is_retail!
        return unless @db.minipet.summon
        return if UnitAffectingCombat "player"
        @summon_battlepet!

      @ace_module\RegisterEvent "PLAYER_ENTERING_WORLD", (...) ->
        return unless idTweaks\is_retail!
        return unless @db.minipet.summon
        return if UnitAffectingCombat "player"
        @summon_battlepet!

      @ace_module\RegisterEvent "PLAYER_MOUNT_DISPLAY_CHANGED", (...) ->
        return unless idTweaks\is_retail!
        return unless @db.minipet.summon
        return if UnitAffectingCombat "player"
        @summon_battlepet!

      @ace_module\RegisterEvent "PLAYER_UNGHOST", (...) ->
        return unless idTweaks\is_retail!
        return unless @db.minipet.summon
        return if UnitAffectingCombat "player"
        @summon_battlepet!

    if idTweaks\is_bc!
      @ace_module\RegisterEvent "UI_ERROR_MESSAGE", (_, _, error_message) ->
        return unless idTweaks\is_bc!
        return unless @db.movement.dismount
        return unless IsMounted!
        
        @dismount! if @mount_error_messages[error_message]

  disable: =>
    -- print "Automation#disable"

    @ace_module\UnregisterEvent "CONFIRM_BINDER"
    @ace_module\UnregisterEvent "CONFIRM_SUMMON"
    @ace_module\UnregisterEvent "MERCHANT_SHOW"
    @ace_module\UnregisterEvent "PLAYER_DEAD"
    @ace_module\UnregisterEvent "RESURRECT_REQUEST"
    
    if idTweaks\is_retail!
      @ace_module\UnregisterEvent "PLAYER_ALIVE"
      @ace_module\UnregisterEvent "PLAYER_CONTROL_GAINED"
      @ace_module\UnregisterEvent "PLAYER_ENTERING_WORLD"
      @ace_module\UnregisterEvent "PLAYER_MOUNT_DISPLAY_CHANGED"
      @ace_module\UnregisterEvent "PLAYER_UNGHOST"

    if idTweaks\is_bc!
      @ace_module\UnregisterEvent "UI_ERROR_MESSAGE"

  summon_battlepet: =>
    -- print "Automation#summon_battlepet"

    C_PetJournal.SummonRandomPet true

  sell_grey_items: =>
    -- print "Automation#sell_grey_items"

    for bag = 0, 4
      for slot = 0, GetContainerNumSlots bag
        link = GetContainerItemLink bag, slot
        if link and (select 3, GetItemInfo link) == 0
          UseContainerItem bag, slot

  repair_gear: =>
    -- print "Automation#repair_gear"

    cost = GetRepairAllCost!
    funds = GetMoney!

    if cost == 0
      @ace_module\Print "Repair: Nothing to repair"

    if funds < cost
      @ace_module\Print "Repair: Insufficient Funds"
      return

    if cost > 0
      @ace_module\Print "Repair: #{GetCoinTextureString cost}"
      RepairAllItems!

  accept_hearthstone_bind: =>
    StaticPopup1Button1\Click!

  accept_summon: =>
    StaticPopup1\Hide!
    C_SummonInfo.ConfirmSummon!

  accept_pvp_release: =>
    in_instance, instance_type = IsInInstance!
    RepopMe! if in_instance and instance_type == "pvp"

  accept_resurrection: =>
    StaticPopup1\Hide!
    AcceptResurrect!

  dismount: =>
    Dismount!

idTweaks\add_module Automation idTweaks
